import sys

from qtpy import QtWidgets

from pyqtribbon import RibbonBar, RibbonCategoryStyle


def test_filemenu():
    if __name__ == '__main__':
        app = QtWidgets.QApplication(sys.argv)
        # Central widget
        window = QtWidgets.QMainWindow()

        # Ribbon bar
        ribbonbar = RibbonBar()
        window.setMenuBar(ribbonbar)

        categories1 = ribbonbar.addCategoriesBy({
            'Category 6': {
                "style": RibbonCategoryStyle.Normal,
                "panels": {
                    "Panel 1": {
                        "showPanelOptionButton": True,
                        "widgets": {
                            "Button 1": {
                                "type": "Button",
                                "arguments": {
                                    "text": "Button",
                                    "tooltip": "This is a tooltip",
                                }
                            },
                            "Button 2": {
                                "type": "Button",
                                "arguments": {
                                    "text": "Button 2",
                                    "tooltip": "This is a tooltip",
                                }
                            },
                        }
                    },
                    "Panel 2": {
                        "showPanelOptionButton": False,
                        "widgets": {
                            "Button 3": {
                                "type": "Button",
                                "arguments": {
                                    "text": "Button 3",
                                    "tooltip": "This is a tooltip",
                                }
                            },
                            "Button 4": {
                                "type": "Button",
                                "arguments": {
                                    "text": "Button 4",
                                    "tooltip": "This is a tooltip",
                                }
                            },
                        }
                    },
                }
            }
        })

        assert categories1['Category 6'].title() == "Category 6"
        assert categories1['Category 6'].categoryStyle() == RibbonCategoryStyle.Normal
        assert categories1['Category 6'].panels()['Panel 1'].title() == "Panel 1"
        assert categories1['Category 6'].panels()['Panel 1']._showPanelOptionButton
        assert categories1['Category 6'].panels()['Panel 1'].widgets()[0].text() == "Button"
        assert categories1['Category 6'].panels()['Panel 1'].widgets()[0].toolTip() == "This is a tooltip"
        assert categories1['Category 6'].panels()['Panel 1'].widgets()[1].text() == "Button 2"
        assert categories1['Category 6'].panels()['Panel 1'].widgets()[1].toolTip() == "This is a tooltip"
        assert categories1['Category 6'].panels()['Panel 2'].title() == "Panel 2"
        assert not categories1['Category 6'].panels()['Panel 2']._showPanelOptionButton
        assert categories1['Category 6'].panels()['Panel 2'].widgets()[0].text() == "Button 3"
        assert categories1['Category 6'].panels()['Panel 2'].widgets()[0].toolTip() == "This is a tooltip"
        assert categories1['Category 6'].panels()['Panel 2'].widgets()[1].text() == "Button 4"
        assert categories1['Category 6'].panels()['Panel 2'].widgets()[1].toolTip() == "This is a tooltip"

        # Show the window
        window.resize(1800, 350)
        window.show()
        sys.exit(app.exec_())



import sys

from qtpy import QtWidgets

from pyqtribbon import RibbonBar
from pyqtribbon.gallery import RibbonGallery


def test_ribbonbar():
    if __name__ == '__main__':
        app = QtWidgets.QApplication(sys.argv)
        # Central widget
        window = QtWidgets.QMainWindow()

        # Ribbon bar
        ribbonbar = RibbonBar()
        window.setMenuBar(ribbonbar)

        category = ribbonbar.addCategory('name')
        panel = category.addPanel('Panel 1')

        gallery = panel.addGallery(popupHideOnClick=True)
        for i in range(100):
            gallery.addToggleButton(f'item {i + 1}')
        popupMenu = gallery.popupMenu()
        submenu = popupMenu.addMenu('Submenu')
        submenu.addAction("Action 1")
        popupMenu.addAction("Action 2")
        popupMenu.addSeparator()
        popupMenu.addWidget(QtWidgets.QLabel("This is a custom widget"))
        formLayout = popupMenu.addFormLayoutWidget()
        formLayout.addRow(QtWidgets.QLabel("Row 1"), QtWidgets.QLineEdit('Line Edit'))

        assert isinstance(gallery, RibbonGallery)
        assert gallery.popupMenu() is not None
        assert gallery.popupMenu().actions()[0].text() == 'Submenu'
        assert submenu.actions()[0].text() == 'Action 1'
        assert gallery.popupMenu().actions()[1].text() == 'Action 2'
        assert gallery.popupMenu().actions()[2].isSeparator()
        assert gallery.popupMenu().actions()[3].defaultWidget().text() == 'This is a custom widget'
        assert isinstance(formLayout, QtWidgets.QFormLayout)
        assert formLayout.itemAt(0, QtWidgets.QFormLayout.LabelRole).widget().text() == 'Row 1'
        assert formLayout.itemAt(0, QtWidgets.QFormLayout.FieldRole).widget().text() == 'Line Edit'

        # Show the window
        window.resize(1800, 350)
        window.show()
        sys.exit(app.exec_())



import sys

from qtpy import QtWidgets

from pyqtribbon import RibbonBar


def test_category():
    if __name__ == '__main__':
        app = QtWidgets.QApplication(sys.argv)
        # Central widget
        window = QtWidgets.QMainWindow()

        # Ribbon bar
        ribbonbar = RibbonBar()
        window.setMenuBar(ribbonbar)

        # Add a category
        category = ribbonbar.addCategory("Category 1")
        assert category.title() == "Category 1"
        assert category in ribbonbar.categories().values()

        # Show the window
        window.resize(1800, 350)
        window.show()
        sys.exit(app.exec_())



import sys

from qtpy import QtGui
from qtpy.QtWidgets import QApplication
from qtpy.QtGui import QIcon

from pyqtribbon import RibbonBar, RibbonCategoryStyle
from pyqtribbon.screenshotwindow import RibbonScreenShotWindow

if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setFont(QtGui.QFont("Times New Roman", 8))
    window = RibbonScreenShotWindow('category.png')

    # Ribbon bar
    ribbonbar = RibbonBar()
    window.setMenuBar(ribbonbar)
    
    # Categories
    category1 = ribbonbar.addCategory('Category 1')
    panel1 = category1.addPanel('Panel 1')
    panel1.addLargeButton('Large Button 1', QIcon('python.png'))
    
    category2 = ribbonbar.addContextCategory('Category 2')
    panel12 = category2.addPanel('Panel 2')
    panel12.addLargeButton('Large Button 2', QIcon('python.png'))

    categories = ribbonbar.addCategoriesBy({
        'Category 6': {
            "style": RibbonCategoryStyle.Normal,
            "panels": {
                "Panel 1": {
                    "showPanelOptionButton": True,
                    "widgets": {
                        "Button 1": {
                            "type": "Button",
                            "arguments": {
                                "icon": QIcon("python.png"),
                                "text": "Button",
                                "tooltip": "This is a tooltip",
                            }
                        },
                    }
                },
            }
        }
    })
    ribbonbar.setCurrentCategory(categories['Category 6'])

    # Show the window
    window.resize(1000, 250)
    window.show()

    sys.exit(app.exec_())



import sys

from qtpy import QtWidgets

from pyqtribbon import RibbonBar
from pyqtribbon.panel import RibbonPanel
from pyqtribbon.toolbutton import RibbonToolButton


def test_categories():
    if __name__ == '__main__':
        app = QtWidgets.QApplication(sys.argv)
        # Central widget
        window = QtWidgets.QMainWindow()

        # Ribbon bar
        ribbonbar = RibbonBar()
        window.setMenuBar(ribbonbar)

        categories = ribbonbar.addContextCategories('name', ['Context 1', 'Context 2'])
        assert isinstance(categories, dict)
        assert len(categories) == 2
        assert categories['Context 1'].title() == 'Context 1'
        assert categories['Context 2'].title() == 'Context 2'

        panel1 = categories['Context 1'].addPanel('Context 1 Panel 1')
        button1 = panel1.addLargeButton('Button 1')
        assert isinstance(panel1, RibbonPanel)
        assert panel1.title() == 'Context 1 Panel 1'
        assert isinstance(button1, RibbonToolButton)
        assert button1.text() == 'Button 1'

        panel2 = categories['Context 2'].addPanel('Context 2 Panel 1')
        button2 = panel2.addLargeButton('Button 2')
        assert isinstance(panel2, RibbonPanel)
        assert panel2.title() == 'Context 2 Panel 1'
        assert isinstance(button2, RibbonToolButton)
        assert button2.text() == 'Button 2'

        # Show the window
        window.resize(1800, 350)
        window.show()
        sys.exit(app.exec_())



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import inspect
import os
import sys
sys.path.insert(0, os.path.abspath('../../'))


# -- Project information -----------------------------------------------------

project = 'pyqtribbon'
copyright = '2022, WANG Hailin'
author = 'WANG Hailin'

# The full version, including alpha/beta/rc tags
import pyqtribbon
release = version = pyqtribbon.__version__


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'numpydoc',
    'sphinx.ext.autodoc',
    'sphinx_toolbox.more_autodoc.overloads',
    'sphinx.ext.linkcode',
    'sphinx.ext.githubpages',
    'sphinx_qt_documentation',
]

qt_documentation = 'Qt5'
autodoc_typehints_format = 'short'
numpydoc_show_inherited_class_members = False


# linkcode source
def linkcode_resolve(domain: str, info: dict):
    """Resolve linkcode source
    Parameters
    ----------
    domain : str
        specifies the language domain the object is in
    info : dict[str, str | list[str]]
        a dictionary with the following keys guaranteed to be present (dependent on the domain)

        - py: module (name of the module), fullname (name of the object)
        - c: names (list of names for the object)
        - cpp: names (list of names for the object)
        - javascript: object (name of the object), fullname (name of the item)

    Returns
    -------
    source url of the object
    """
    if domain != 'py':
        return None

    modname = info['module']
    fullname = info['fullname']

    filename = modname.replace('.', '/')
    baseurl = f'https://github.com/haiiliin/pyqtribbon/blob/main/{filename}.py'

    submod = sys.modules.get(modname)
    if submod is None:
        return baseurl

    obj = submod
    for part in fullname.split('.'):
        try:
            obj = getattr(obj, part)
        except Exception:
            return baseurl
    try:
        source, lineno = inspect.getsourcelines(obj)
    except Exception:
        return baseurl

    return baseurl + f'#L{lineno}-L{lineno + len(source) - 1}'


# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']



import sys

from PyQt5.QtWidgets import QApplication, QLabel, QWidget, QVBoxLayout
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtCore import Qt

from pyqtribbon import RibbonBar
from pyqtribbon.screenshotwindow import RibbonScreenShotWindow
from pyqtribbon.utils import data_file_path

if __name__ == "__main__":
    app = QApplication(sys.argv)
    app.setFont(QFont("Times New Roman", 8))
    
    # Central widget
    window = RibbonScreenShotWindow('tutorial-ribbonbar.png')
    window.setWindowIcon(QIcon(data_file_path("icons/python.png")))
    centralWidget = QWidget()
    window.setCentralWidget(centralWidget)
    layout = QVBoxLayout(centralWidget)
    
    # Ribbon bar
    ribbonbar = RibbonBar()
    window.setMenuBar(ribbonbar)
    category = ribbonbar.addCategory("Category 1")
    panel = category.addPanel("Panel 1")
    panel.addLargeButton("A Large Button", QIcon(data_file_path("icons/python.png")))
    panel.addMediumButton("A Medium Button", QIcon(data_file_path("icons/python.png")))
    panel.addMediumButton("A Medium Button", QIcon(data_file_path("icons/python.png")))
    panel.addSmallButton("A Small Button", QIcon(data_file_path("icons/python.png")))
    panel.addSmallButton("A Small Button", QIcon(data_file_path("icons/python.png")))
    panel.addSmallButton("A Small Button", QIcon(data_file_path("icons/python.png")))
    
    # Display a label in the main window
    label = QLabel("Ribbon Test Window")
    label.setFont(QFont("Arial", 20))
    label.setAlignment(Qt.AlignCenter)
    
    # Add the ribbon bar and label to the layout
    layout.addWidget(label, 1)
    
    # Show the window
    window.resize(1800, 350)
    window.show()
    sys.exit(app.exec_())


from .category import RibbonCategoryStyle, Normal, Context
from .menu import RibbonMenu, RibbonPermanentMenu
from .panel import RibbonSpaceFindMode, ColumnWise, RowWise
from .ribbonbar import RibbonBar, RibbonStyle, Default, Debug
from .toolbutton import RibbonButtonStyle, Small, Medium, Large

from .version import __version__



import sys

from qtpy import QtWidgets

from pyqtribbon import RibbonBar


def test_filemenu():
    if __name__ == '__main__':
        app = QtWidgets.QApplication(sys.argv)
        # Central widget
        window = QtWidgets.QMainWindow()

        # Ribbon bar
        ribbonbar = RibbonBar()
        window.setMenuBar(ribbonbar)

        fileMenu = ribbonbar.applicationOptionButton().addFileMenu()
        submenu = fileMenu.addMenu("Submenu")
        action1 = submenu.addAction("Action 1")
        assert action1.text() == "Action 1"
        assert action1 in submenu.actions()

        fileMenu.addSeparator()

        action2 = fileMenu.addAction("Action 2")
        assert action2.text() == "Action 2"
        assert action2 in fileMenu.actions()

        # Show the window
        window.resize(1800, 350)
        window.show()
        sys.exit(app.exec_())



import sys

from qtpy import QtWidgets

from pyqtribbon import RibbonBar


def test_panel():
    if __name__ == '__main__':
        app = QtWidgets.QApplication(sys.argv)
        # Central widget
        window = QtWidgets.QMainWindow()

        # Ribbon bar
        ribbonbar = RibbonBar()
        window.setMenuBar(ribbonbar)

        # Add a category
        category = ribbonbar.addCategory("Category 1")
        assert category.title() == "Category 1"
        assert category in ribbonbar.categories().values()

        # Add a panel
        panel = category.addPanel("Panel 1")
        assert panel.title() == "Panel 1"
        assert panel in category.panels().values()

        button1 = panel.addButton("Button 1")
        assert button1.text() == "Button 1"
        assert button1 in panel.widgets()
        button2 = panel.addButton("Button 2")
        assert button2.text() == "Button 2"
        assert button2 in panel.widgets()

        # Show the window
        window.resize(1800, 350)
        window.show()
        sys.exit(app.exec_())
